#![feature(async_closure)]

use std::{path::{Path, PathBuf}, sync::mpsc, thread::JoinHandle};
use windows::WindowId;
use xmpd_manifest::{store::JsonStore, Manifest};

mod main_window;
mod windows;


const W_NAME: &str = "xmpd v2.0.0a";

type Result<T> = anyhow::Result<T>;

pub fn start(manifest_path: PathBuf) -> Result<()> {
    let options = eframe::NativeOptions::default();
    let mut state = GuiState::new(&manifest_path)?;
    state.windows.toggle(&WindowId::Error, true);
    
    let res = eframe::run_simple_native(W_NAME, options, move |ctx, _frame| {
        state.windows.clone().draw_all(ctx, &mut state);
        egui::CentralPanel::default().show(ctx, |ui| main_window::draw(ui, &mut state));
        ctx.request_repaint(); 
    });
    if let Err(e) = res { // dumb err value by eframe
        anyhow::bail!(e.to_string());
    }
    Ok(())
}


pub enum Message {
    
}

pub struct GuiState {
    pub manifest: Manifest<JsonStore>,
    pub windows: windows::Windows
}

impl GuiState {
    pub fn new(manifest_path: &Path) -> Result<Self> {
        Ok(Self {
            manifest: Manifest::new(manifest_path)?,
            windows: windows::Windows::new(),
        })
    }
}



