use std::{collections::HashMap, path::Path};

use uuid::Uuid;

use crate::{playlist::Playlist, song::Song};

mod json;
pub use json::JsonStore;

pub trait BaseStore { 
    fn get_songs(&self) -> &HashMap<Uuid, Song>;
    fn get_songs_mut(&mut self) -> &mut HashMap<Uuid, Song>;
    fn get_song(&self, id: &Uuid) -> Option<&Song>;
    fn get_song_mut(&mut self, id: &Uuid) -> Option<&mut Song>;

    fn get_playlists(&self) -> &HashMap<Uuid, Playlist>;
    fn get_playlists_mut(&mut self) -> &mut HashMap<Uuid, Playlist>;
    fn get_playlist(&self, id: &Uuid) -> Option<&Playlist>;
    fn get_playlist_mut(&mut self, id: &Uuid) -> Option<&mut Playlist>;

    fn to_bytes(&self) -> crate::Result<Vec<u8>>;
    fn from_bytes(s: &[u8]) -> crate::Result<Self> where Self: Sized;
    fn empty() -> Self where Self: Sized;
    fn save_to(&self, p: &Path) -> crate::Result<()> {
        let bin = self.to_bytes()?;
        std::fs::write(p, bin)?;
        Ok(())
    }
    fn load_from(&mut self, p: &Path) -> crate::Result<()> where Self: Clone {
        let bin = std::fs::read(p)?;
        let s = Self::from_bytes(&bin)?;
        self.clone_from(&s);
        Ok(())
    }
    fn get_default_file_contents() -> &'static str;
    fn get_file_extension() -> &'static str;
    fn save_original_path(&mut self, p: &Path);
    fn get_original_path(&self) -> &Path;
}

